Ecto.NoPrimaryKeyFieldError exception (Ecto v3.14.0)

Copy Markdown View Source

Raised at runtime when an operation that requires a primary key is invoked with a schema that does not define a primary key by using @primary_key false