Ecto.UUID (Ecto v3.14.0)

Copy Markdown View Source

An Ecto type for UUID strings.

Autogeneration

This type can be used for any UUID field in your schemas. It is used when autogenerating binary IDs in Ecto.Schema. By default, autogenerated UUIDs use version 4 (random):

use Ecto.Schema
@primary_key {:id, :binary_id, autogenerate: true}

To use UUID v7 (time-ordered) instead:

use Ecto.Schema
@primary_key {:id, Ecto.UUID, autogenerate: [version: 7]}

To use UUID v7 (time-ordered) monotonic:

use Ecto.Schema
@primary_key {:id, Ecto.UUID, autogenerate: [version: 7, precision: :monotonic]}

According to RFC 9562: "Monotonicity (each subsequent value being greater than the last) is the backbone of time-based sortable UUIDs."

Summary

Types

Supported options: :version and :precision (v7-only).

A raw binary representation of a UUID.

t()

A hex-encoded UUID string.

Functions

Generates a uuid with the given options in binary format. See generate/1 for details and available options.

Casts either a string in the canonical, human-readable UUID format or a 16-byte binary to a UUID in its canonical, human-readable UUID format.

Same as cast/1 but raises Ecto.CastError on invalid arguments.

Converts a string representing a UUID into a raw binary.

Same as dump/1 but raises Ecto.ArgumentError on invalid arguments.

Callback implementation for Ecto.Type.embed_as/1.

Callback implementation for Ecto.Type.equal?/2.

Generates a UUID string.

Converts a binary UUID into a string.

Same as load/1 but raises Ecto.ArgumentError on invalid arguments.

Types

option()

@type option() :: {:version, 4 | 7} | {:precision, :millisecond | :monotonic}

Supported options: :version and :precision (v7-only).

options()

@type options() :: [option()]

raw()

@type raw() :: <<_::128>>

A raw binary representation of a UUID.

t()

@type t() :: <<_::288>>

A hex-encoded UUID string.

Functions

bingenerate(opts \\ [])

@spec bingenerate(options()) :: raw()

Generates a uuid with the given options in binary format. See generate/1 for details and available options.

cast(uuid)

@spec cast(t() | raw() | any()) :: {:ok, t()} | :error

Casts either a string in the canonical, human-readable UUID format or a 16-byte binary to a UUID in its canonical, human-readable UUID format.

If uuid is neither of these, :error will be returned.

Since both binaries and strings are represented as binaries, this means some strings you may not expect are actually also valid UUIDs in their binary form and so will be casted into their string form.

If you need further-restricted behavior or validation, you should define your own custom Ecto.Type. There is also Ecto.UUID.load/1 if you only want to process raw UUIDs, which may be a more suitable reverse operation to Ecto.UUID.dump/1.

Examples

iex> Ecto.UUID.cast(<<0x60, 0x1D, 0x74, 0xE4, 0xA8, 0xD3, 0x4B, 0x6E,
...>                  0x83, 0x65, 0xED, 0xDB, 0x4C, 0x89, 0x33, 0x27>>)
{:ok, "601d74e4-a8d3-4b6e-8365-eddb4c893327"}

iex> Ecto.UUID.cast("601d74e4-a8d3-4b6e-8365-eddb4c893327")
{:ok, "601d74e4-a8d3-4b6e-8365-eddb4c893327"}

iex> Ecto.UUID.cast("warehouse worker")
{:ok, "77617265-686f-7573-6520-776f726b6572"}

cast!(uuid)

@spec cast!(t() | raw() | any()) :: t()

Same as cast/1 but raises Ecto.CastError on invalid arguments.

dump(uuid_string)

@spec dump(uuid_string :: t() | any()) :: {:ok, raw()} | :error

Converts a string representing a UUID into a raw binary.

dump!(uuid)

@spec dump!(t() | any()) :: raw()

Same as dump/1 but raises Ecto.ArgumentError on invalid arguments.

embed_as(_)

Callback implementation for Ecto.Type.embed_as/1.

equal?(term1, term2)

Callback implementation for Ecto.Type.equal?/2.

generate(opts \\ [])

@spec generate(options()) :: t()

Generates a UUID string.

Options

  • :version - The UUID version to generate. Supported values are 4 (random) and 7 (time-ordered). Defaults to 4.

Options (version 7 only)

  • :precision - The timestamp precision for version 7 UUIDs. Supported values are :millisecond and :monotonic. Defaults to :millisecond.

Monotonic precision

When using :monotonic, sub-millisecond precision is encoded in the rand_a field. The generated version 7 UUIDs are strictly monotonically increasing (per node), even when multiple UUIDs are generated within the same timestamp. This is useful for maintaining insertion order in databases.

Examples

> Ecto.UUID.generate()
"a0eebc99-9c0b-4ef8-bb6d-6bb9bd380a11"

> Ecto.UUID.generate(version: 7)
"018ec4c1-ae46-7f5a-8f5a-6f5a8f5a6f5a"

> Ecto.UUID.generate(version: 7, precision: :millisecond)
"018ec4c1-ae46-7f5a-8f5a-6f5a8f5a6f5a"

> Ecto.UUID.generate(version: 7, precision: :monotonic)
"018ec4c1-ae46-7f5a-8f5a-6f5a8f5a6f5a"

load(raw_uuid)

@spec load(raw() | any()) :: {:ok, t()} | :error

Converts a binary UUID into a string.

load!(value)

@spec load!(raw() | any()) :: t()

Same as load/1 but raises Ecto.ArgumentError on invalid arguments.