Mix.Tasks.Ecto.Create
Create the database for the given repository, as specified in the repo's url.
Command line options
--no-start- do not start applications
Examples
mix ecto.create MyApp.Repo
Create the database for the given repository, as specified in the repo's url.
--no-start - do not start applicationsmix ecto.create MyApp.Repo