Mix.Tasks.Ecto.Drop
Drop the database for the given repository, as specified in the repo's url.
Command line options
--no-start- do not start applications
Examples
mix ecto.drop MyApp.Repo
Drop the database for the given repository, as specified in the repo's url.
--no-start - do not start applicationsmix ecto.drop MyApp.Repo